@@ -96,7 +96,7 @@ class Lexer(object):
         "SORTBY",
         "ASCDESC",
 
-        "NUMBER",
+        "DIGITS",
         "REGEX",
         # "EQUAL",
 
@@ -123,7 +123,7 @@ class Lexer(object):
 
     REQUIRES_INDENT = ("IF", "ELSEIF", "ELSE", "CHECKOUT", "SUITE")
 
-    NUMBER = r"""\-?\$?(?:0[xX][0-9a-fA-F]+|0[0-7]+|(?:(?:0|[1-9][0-9]*)\.[0-9]*(?:[eE][+-]?[0-9]+)?|\.[0-9]+(?:[eE][+-]?[0-9]+)?|(?:0|[1-9][0-9]*)(?:[eE][+-]?[0-9]+)?))\%?"""
+    DIGITS = r"""\-?\$?(?:0[xX][0-9a-fA-F]+|0[0-7]+|(?:(?:0|[1-9][0-9]*)\.[0-9]*(?:[eE][+-]?[0-9]+)?|\.[0-9]+(?:[eE][+-]?[0-9]+)?|(?:0|[1-9][0-9]*)(?:[eE][+-]?[0-9]+)?))\%?"""
 
     # Token
     t_PLUS = r'\+'
@@ -223,8 +223,8 @@ class Lexer(object):
         t.value = t.value[1:-1]
         return t
 
-    @TOKEN(NUMBER)
-    def t_NUMBER(self, t):
+    @TOKEN(DIGITS)
+    def t_DIGITS(self, t):
         if t.value.endswith('%'):
             t.value = float(t.value[:-1]) / 100
         elif t.value.find('$') > -1:
