@@ -85,10 +85,9 @@ class Parser(object):
         '''varpaths : VARPATH
                     | varpaths VARPATH'''
         if len(p) == 2:
-            p[0] = [p[1]]
+            p[0] = ast.Varpath(varpath=p[1])
         else:
-            p[1].append(p[2])
-            p[0] = p[1]
+            p[0] = p[1].add(varpath=p[2])
 
     def p_task_bfs(self, p):
         '''task : AFTER CHECKOUT suite EOF
@@ -203,8 +202,7 @@ class Parser(object):
         '''else_if : ELSEIF expressions suite
                    | else_if ELSEIF expressions suite'''
         if len(p) == 4:
-            p[0] = [ast.Elseif(parser=self, lineno=p.lineno(1), expressions=p[2], suite=p[3])
-                   ]
+            p[0] = [ast.Elseif(parser=self, lineno=p.lineno(1), expressions=p[2], suite=p[3])]
         else:
             p[1].append(ast.Elseif(parser=self, lineno=p.lineno(2), expressions=p[3], suite=p[4]))
             p[0] = p[1]
@@ -286,8 +284,11 @@ class Parser(object):
         p[0] = ast.Quit()
     
     def p_foreach(self, p):
-        '''stmt : FOREACH varpaths suite'''
-        p[0] = ast.Foreach(varpath=ast.Varpath(varpath=p[2]),
+        '''stmt : FOREACH expression suite
+                | FOREACH setter suite'''
+        p[0] = ast.Foreach(parser=self,
+                           lineno=p.lineno(1),
+                           this=p[2],
                            suite=p[3])
 
     # ----------------------
@@ -334,8 +335,10 @@ class Parser(object):
     def p_varexp_agg(self, p):
         '''agg : AVERAGE
                | COUNT
+               | AVG
                | NUMBER OF
                | MAX
+               | SUM
                | MIN
                | SMALLEST
                | NEWEST
@@ -356,10 +359,10 @@ class Parser(object):
             p[0] = p[2]
 
     def p_varexp(self, p):
-        '''expression : agg offset randunique expression WHERE expressions SORTBY varpaths ASCDESC time
-                      | agg offset randunique expression SORTBY varpaths ASCDESC time
-                      | agg offset randunique expression WHERE expressions time
-                      | agg offset randunique expression time'''
+        '''expression : agg offset randunique expressions WHERE expressions SORTBY varpaths ASCDESC time
+                      | agg offset randunique expressions SORTBY varpaths ASCDESC time
+                      | agg offset randunique expressions WHERE expressions time
+                      | agg offset randunique expressions time'''
 
         if len(p) == 11:
             _ = dict(where=p[6], sortby=(p[8], p[9]), time=p[10])
@@ -376,8 +379,27 @@ class Parser(object):
                                       agg=p[1],
                                       **_)
 
+    def p_varexp_simple(self, p):
+        '''setter : varpaths WHERE expressions SORTBY varpaths ASCDESC time
+                  | varpaths SORTBY varpaths ASCDESC time
+                  | varpaths WHERE expressions time
+                  | varpaths time'''
+
+        if len(p) == 8:
+            _ = dict(where=p[3], sortby=(p[5], p[6]), time=p[7])
+        elif len(p) == 6:
+            _ = dict(sortby=(p[3], p[4]), time=p[5])
+        elif len(p) == 5:
+            _ = dict(where=p[3], time=p[4])
+        elif len(p) == 3:
+            _ = dict(time=p[2])
+
+        p[0] = ast.AdvancedExpression(expression=ast.Expression(expression=p[1]),
+                                      **_)
+
     def p_stmt_set_varpath(self, p):
-        '''stmt : SET VARPATH TO expressions NEWLINE'''
+        '''stmt : SET VARPATH TO expressions NEWLINE
+                | SET VARPATH TO setter NEWLINE'''
         p[0] = ast.Set(parser=self, lineno=p.lineno(1),
                        varpath=p[2], value=p[4])
 
@@ -393,14 +415,14 @@ class Parser(object):
     def p_stmt_with(self, p):
         '''stmt : WITH varpaths suite'''
         p[0] = ast.With(parser=self, lineno=p.lineno(1),
-                        varpath=ast.Varpath(varpath=p[2]), suite=p[3])
+                        varpath=p[2], suite=p[3])
 
     # ------------------
     # Expressions
     # ------------------
     def p_expression_varpath(self, p):
         '''expression : varpaths'''
-        p[0] = ast.Expression(expression=ast.Varpath(varpath=p[1]))
+        p[0] = ast.Expression(expression=p[1])
 
     def p_expression_num(self, p):
         '''expression : DIGITS'''
@@ -425,7 +447,7 @@ class Parser(object):
     def p_expression_has(self, p):
         '''expression : varpaths HAS VARPATH'''
         assert not p[3].endswith(']'), "Index lookup must not be a list"
-        p[0] = ast.Expression(expression=ast.Method(left=ast.Varpath(p[1]),
+        p[0] = ast.Expression(expression=ast.Method(left=p[1],
                                                     method="has",
                                                     right=ast.Varpath(p[3])))
 
@@ -433,13 +455,13 @@ class Parser(object):
         '''expression : VARPATH IN varpaths'''
         p[0] = ast.Expression(expression=ast.Method(left=ast.Varpath(varpath=p[1]),
                                                     method="in",
-                                                    right=ast.Varpath(varpath=p[3])))
+                                                    right=p[3]))
 
     def p_expression_contains(self, p):
-        '''expression : expression CONTAINS VARPATH'''
-        p[0] = ast.Expression(expression=ast.Method(varpath=p[1],
+        '''expression : expression CONTAINS variable'''
+        p[0] = ast.Expression(expression=ast.Method(left=p[1],
                                                     method="contains",
-                                                    other=ast.Varpath(varpath=p[3])))
+                                                    right=p[3]))
 
     def p_expression_likere(self, p):
         '''expression : expression LIKE REGEX'''
