@@ -54,7 +54,9 @@ KEYWORDS = [
   "oldest",
   "count",
   "average",
+  "avg",
   "max",
+  "sum",
   "min",
   "number",
   
